#########
#
# LOGREP - Log reporting system common library
# v1.3 - Generate 2D links even if there is only one element, sep 02, TK
# v1.2 - Make sure that graph elements fewer than 10 are treated properly, sep 02, TK
# v1.1 - Use only basename for html links, jun 02, TK
# v1.0 - Initial version, jun 02, Tevfik K. (www.itefix.no)

use CGI qw/*table/;
use GD::Graph::bars;
use Getopt::Std;

%monthnr = (
	"Jan" => "01",
	"Feb" => "02",
	"Mar" => "03",
	"Apr" => "04",
	"May" => "05",
	"Jun" => "06",
	"Jul" => "07",
	"Aug" => "08",
	"Sep" => "09",
	"Oct" => "10",
	"Nov" => "11",
	"Dec" => "12"
);

$dieopenmsg = "Can't open file $!";
$dieclosemsg = "Can't close file $!";

# Sets common parameters for reporting modules
#
sub SetCommonParameters {
	
	$q = new CGI;
	$indexpage = "index.html";
	$cssfile = "logrep.css";

	# Get command line parameters
	getopts("l:w:") or die $!;
	$webdir = $opt_w;
	$logfilename = $opt_l;
	
	# Generate table names
	foreach my $x (@dim) {
		foreach my $y (@dim) {	
			next if ($x eq $y);
			$tblname {"$x$y"} = "tbl$x$y";
			$resname {"$x$y"} = ucfirst($x) . "-" . ucfirst($y);	
		}
	}
	
	$startdate = $enddate = "";
		
	# Generate style sheets
	GenerateCSSFile ();
}

# Generates stylesheet for web pages
#
sub GenerateCSSFile {

	open HTMLPAGE, "> $webdir/$cssfile" or die $dieopenmsg;
	
	print HTMLPAGE "A:hover {color : Green;}\n";
	
	print HTMLPAGE "H1 {font-family : Verdana, Geneva, Arial, Helvetica, sans-serif; font-size : x-large;}\n";
	print HTMLPAGE "H2 {font-family : Verdana, Geneva, Arial, Helvetica, sans-serif; font-size : large;}\n";
	print HTMLPAGE "H3 {font-family : Verdana, Geneva, Arial, Helvetica, sans-serif; font-size : medium;}\n";
	print HTMLPAGE "BODY {font-family : Verdana, Geneva, Arial, Helvetica, sans-serif;	font-size : 12; color : #000080; background-color : #FFA07A;}\n";
	print HTMLPAGE "TABLE {font-family : Verdana, Geneva, Arial, Helvetica, sans-serif; font-size : 12; color : #000080;}\n";

	close HTMLPAGE or die $dieclosemsg;
}

# Generates counter tables
#
sub GenerateTables {
	foreach my $x (@dim) {
		foreach my $y (@dim) {			
			next if ($x eq $y);						
			my $table = $tblname {"$x$y"};
			$$table{$$x}{$$y} = (exists $$table{$$x}{$$y}) ? ($$table{$$x}{$$y}) + 1 : 1;
			$$table{$$x}{"sum"} = (exists $$table{$$x}{"sum"}) ? ($$table{$$x}{"sum"}) + 1 : 1;				
		}
	}
}

# WriteIndexPage : 
#	Generates index html page for analysis results
#	Parameter 1 : Main title
#	Parameter 2 : Title for parameters table
#	parameter pair (can be more than one): summary title, summary value
#
sub WriteIndexPage {

	my $summarytitle = shift;
	my $tabletitle = shift;
	
	open HTMLPAGE, "> $webdir/$indexpage" or die $dieopenmsg;
	
	PrintHtmlHeader (HTMLPAGE);	

	print HTMLPAGE
		$q->h1 ($summarytitle),
		$q->p (	
			$q->b ("Filename: "),
			"$logfilename ($n lines, from $startdate to $enddate)"
		);
		
	foreach $summary (@_) {

		my $summarytitle = shift;
		my $summaryvalue = shift;
		
		print HTMLPAGE
			$q->p (
				$q->b ($summarytitle),
				$summaryvalue
			);
	}

	# Print html code for parameter table	
	print HTMLPAGE 
		$q->hr, 
		$q->h2 ($tabletitle);

	print HTMLPAGE '<table border="1">';

	foreach my $x (@dim) {

		print HTMLPAGE "<TR>";
	
		foreach my $y (@dim) {
		
			next if ($x eq $y);
		
			my $htmlpagex = $resname{"$x$y"} . "-x.html";
		
			# Don't generate link if there is only one element
			my $table = $tblname {"$x$y"};
			my $nkeys = keys %$table;
			my $link = ($nkeys > 1) 
					? $q->a ({href=>"$htmlpagex"}, $resname{"$x$y"})
					: $q->font({color=>"808080"},$resname{"$x$y"});

			print HTMLPAGE 
				$q->td (
					$q->div ({align=>"center"}, 
						$q->h3 ($link)
					)
				);	
		}
		
		print HTMLPAGE "</TR>";
	}

	print HTMLPAGE $q->end_table;
	
	print HTMLPAGE $q->end_html;
	
	close HTMLPAGE or die $dieclosemsg;
}

###############################
# WriteDetailsPages : 
#	Generates html pages for details
#
sub WriteDetailsPages {
	
	foreach my $x (@dim) {
		foreach my $y (@dim) {
		
			# Not interested in same parameters in both dimensions
			next if ($x eq $y);
			
			# Don't generate file if there is only one element
			my $table = $tblname {"$x$y"};
			my $nkeys = keys %$table;
			next if ($nkeys <= 1);
		
			# Generate file names for details pages
			my $basenamex = $resname{"$x$y"} . "-x.html";
			my $basenamey = $resname{"$x$y"} . "-y.html";
			my $htmlpagex = "$webdir/$basenamex" ;
			my $htmlpagey = "$webdir/$basenamey";
			
			open HTMLPAGEX, "> $htmlpagex" or die $dieopenmsg;
			open HTMLPAGEY, "> $htmlpagey" or die $dieopenmsg;
			
			PrintHtmlHeader (HTMLPAGEX);
			PrintHtmlHeader (HTMLPAGEY);
			
			# Generate html output for Top 10 graph
			print HTMLPAGEX 
				$q->h2 (
					"Top 10 graph for ",
					$q->font ( {color=>"#800000"}, 
						$q->em (ucfirst ($x))
					)
				),
				$q->hr,
				$q->p (
					$q->img ({src=>"$x$y.png",align=>"CENTER"})
				), 
				$q->hr,
				$q->h2 (
					"Details for ",
					$q->font ( {color=>"#800000"}, 
						$q->em (ucfirst ($x))
					),
					" and Links for ",
					$q->font ( {color=>"#800000"}, 
						$q->em (ucfirst ($y))
					)
				),
				'<table border="1"',
				"<tr>",
					$q->td ($q->b ("$x")),
					$q->td ($q->b ("Count/Link")),
				"</tr>";
			
			# Initialize datastructures for graph
			$graphxn = 0;
			$graphdata = undef;
				
			# Generate html output for table entries sorted by sum of occurences
			foreach $$x (sort { $$table{$b}{"sum"} <=> $$table{$a}{"sum"} } keys %$table) 	{		
		
				my $sum = $$table{$$x}{"sum"};
				
				# Populate counters for Top 10 graph
				if ($graphxn < 10) {
					$graphdata [0][$graphxn] = $$x;
					$graphdata [1][$graphxn] = $sum;
					$graphxn++;
				};
				
				my $link = $q->a ({href=>"$basenamey#$$x"}, $sum);
				
				print HTMLPAGEX 
					"<tr>", 
						$q->td ("$$x"), 
						$q->td ($link),
					"</tr>";

				print HTMLPAGEY 
					$q->h2 (
						$q->a ({name=>"$$x"},
							"Details for ",
							$q->font ( {color=>"#800000"}, 
								$q->em (ucfirst ($x))
							),
							" - $$x"
						),
					),
					'<table border="1"',
					"<tr>",
						$q->td ($q->b ("$y")),
						$q->td ($q->b ("Count")),
					"</tr>";
				
				foreach $$y (sort  { $$table{$$x}{$b} <=> $$table{$$x}{$a} } keys %{$$table{$$x}}) {				 
					
					print HTMLPAGEY 
						"<tr>", 
							$q->td ("$$y"),
							$q->td ("$$table{$$x}{$$y}"),
						"</tr>";
				}
				
				print HTMLPAGEY	
					$q->end_table,
					$q->hr;
			}
			
			#Generate graph
			
			# Make sure that graph elements fewer than 10 are treated properly
			while ($graphxn < 10) {
			
				$graphdata [0][$graphxn] = "";
				$graphdata [1][$graphxn] = "";
				$graphxn++;
				
			}
			
			$graph = GD::Graph::bars->new(400, 400);
			$graph->set( 
				x_label => "$x",
				x_labels_vertical => "1",
				y_label => "Count", 
				title => "$x - Top 10", 
				dclrs => ["dblue"]);
			my $gd = $graph->plot(\@graphdata);
			open(IMG, ">$webdir/$x$y.png") or die $!; 
			binmode IMG; 
			print IMG $gd->png if (defined $gd);
			
			print HTMLPAGEX	$q->end_html;
			
			print HTMLPAGEY 
				$q->end_table,
				$q->end_html;
		
			close HTMLPAGEX or die $dieclosemsg;
			close HTMLPAGEY or die $dieclosemsg;
		}
	}
	
}

# PrintHtmlHeader : 
#	Prints a standard html heeader
#	Parameter : filehandle of web page
#
sub PrintHtmlHeader {

	my $fn = shift;
	print $fn $q->start_html(
		-title=>"Report system for logfiles",
		-style=>{"src" => $cssfile}
	);

}
