#########
#
# Trend Micro VirusWall module for log reporting system
# v1.0 - Initial version, 06.2002, Tevfik K. (www.itefix.no)

# Sets analysis parameters - specific for each module
@dim = ( "month", "day", "hour", "from", "to", "virus");

# Get common functions & parameters
do "lib1.pl";

################
# MAIN PROGRAM #
################

SetCommonParameters ();

# initialize counters
$nvirus = $nsmtp = $nftp = $nhttp = 0;

ProcessVirusWallLogFile ($logfilename);

WriteIndexPage (
	"VirusWall Logfile analysis",
	"Virus Analysis Table",
	"Number of viruses: ", "$nvirus (SMTP: $nsmtp, HTTP:$nhttp, FTP:$nftp)",
	);

WriteDetailsPages ();

###############
# SUBROUTINES #
###############

# ProcessVirusWallLogFile : 
#	Processes VirusWall logfile and populates counter arrays
#	Parameter : VirusWall log filename
#
sub ProcessVirusWallLogFile {

	my $viruswallfile = shift;
		
	open LOGFILE, $viruswallfile or die $dieopenmsg;

LOOP: 
	while (<LOGFILE>) {
		
		my $line = $_;
		($month, $day, $hour, $minute) = ($line =~ m"Date:.*(\d\d)/(\d\d)/\d\d\d\d (\d\d):(\d\d)");
		
		($startdate = "$month/$day $hour:$minute") if ($. == 1);
		
		$line = <LOGFILE>;
		($method) = ($line =~ /Method:\s*(\w*)/);
		SW1: {
			if (uc($method) eq "SMTP") 	{$nsmtp++; last SW1 }
			if (uc($method) eq "HTTP") 	{$nhttp++; last SW1 }
			if (uc($method) eq "FTP") 	{$nftp++; last SW1 }
		}

		$line = <LOGFILE>;
		($from) = ($line =~ /From:\s*(\S*)/);
		
		$line = <LOGFILE>;
		($to) = ($line =~ /To:\s*(\S*)/);
		
		<LOGFILE>;
		<LOGFILE>;
		
		$line = <LOGFILE>;
		($virus) = ($line =~ /Virus:\s*(\w*)/);
		
		<LOGFILE>;
					
		#DEBUG! print "$. " if  ( ( ($./100) - int($./100) ) == 0) ;

		$nvirus++;
				
		#DEBUG! print "$month $day $hour $method $from $to $virus\n";
		
		GenerateTables ();
	}
	
	$enddate = "$month/$day $hour:$minute";

	$n = $.;
	
	close LOGFILE or die $dieclosemsg;
	
	#DEBUG! print "\n$n $nvirus $nsmtp $nftp $nhttp\n";
}
