#########
#
# NT Event Log module for log reporting system
# v1.0 - Initial version, Sep 2002, Tevfik K. (tevfik@itefix.no)

# Sets analysis parameters - specific for each module

@dim = ("day", "month", "hour", "type", "category", "event", "source", "user");

# Get common functions & parameters
do "lib1.pl";

%typetxt = (
	"0" => "Success",
	"1" => "Error",
	"2" => "Warning",
	"4" => "Information",
	"8" => "Audit success",
	"10" => "Audit Failure"
);

################
# MAIN PROGRAM #
################

SetCommonParameters ();

# initialize counters
$n = $nerror = $nwarning = 0;

ProcessNTEventLogFile ($logfilename);

WriteIndexPage (
	"NT Event Log analysis",
	"Event Analysis Table",
	"Number of errors:", "$nerror",
	"Number of warnings:", "$nwarning"
	);

WriteDetailsPages ();

###############
# SUBROUTINES #
###############

# ProcessNTEventLogFile : 
#	Processes NT eventlogs produced by 'dumpel' and populates counter arrays
#	Parameter : filename
#
sub ProcessNTEventLogFile {

	my $logfile = shift;
	
	open LOGFILE, $logfile or die $dieopenmsg;

LOOP: 
	while (<LOGFILE>) {

		my $line = $_;

		($month, $day, $hour, $minute, $ntype, $category, $event, $source, $user) =
		($line =~ m!(\d+)\/(\d+)\/.+\t(\d+):(\d+):.+\t(\d+)\t(\d+)\t(\d+)\t([\ \S]+)\t(\S+)!) ;

		($startdate = "$month/$day $hour:$minute") if ($. == 1);
		
		$nerror++ if ($ntype == 1);
		$nwarning++ if ($ntype == 2);
				
		print "$. " if  ((($./5000) - int($./5000) ) == 0) ;
		
		# Type as string
		$type = $typetxt {$ntype};
		
		#DEBUG! print "$month, $day, $hour, $minute, $type, $category, $event, $source, $user\n";
		
		GenerateTables ();
	}
	
	$enddate = "$month/$day $hour:$minute";

	$n = $.;
	
	close LOGFILE or die $dieclosemsg;
}
