#########
#
# Postfix module for log reporting system
# v1.0 - Initial version, 06.2002, Tevfik K. (www.itefix.no)

# Sets analysis parameters - specific for each module
@dim = ( "month", "day", "hour", "from", "to", "size", "recipients", "delay", "status" );

# Get common functions & parameters
do "lib1.pl";

################
# MAIN PROGRAM #
################

SetCommonParameters ();

# initialize counters
$nskipped = $nsmtpd = $ncleanup = $pickup = $nqmgr = $nsmtp = $nmessages = 0;

ProcessPostfixLogFile ($logfilename);

WriteIndexPage (
	"POSTFIX Logfile analysis",
	"Messages Analysis Table",
	"Number of messages processed: ", "$nmessages"
);

WriteDetailsPages ();

###############
# SUBROUTINES #
###############

# ProcessPostfixLogFile : 
#	Processes postfix logfile and populates counter arrays
#	Parameter : Postfix log filename
#
sub ProcessPostfixLogFile {

	my $postfixfile = shift;
		
	open LOGFILE, $postfixfile or die "can't open postfix logfile";

LOOP: 
	while (<LOGFILE>) {

		$line = $_;
		
		print "$. " if  ( ( ($./5000) - int($./5000) ) == 0) ;
		
		#Get linetype
		($linetype) = ($line =~ /.*postfix\/(\w*)\[/);
		#DEBUG! print "$linetype ";
		
		($monthmmm, $day, $hour, $minute) = ($line =~ /(\w\w\w) (\d\d) (\d\d):(\d\d)/);
		
		$month = $monthnr {$monthmmm};
		
		($startdate = "$month/$day $hour:$minute") if ($. == 1);		
				
		SW1 : {
		
			if ( $linetype eq "smtpd" ) { $nsmtpd++; next LOOP }
			
			if ( $linetype eq "cleanup") { $ncleanup++; next LOOP }
						
			if ( $linetype eq "pickup") { 
			
				$npickup++;
				# Pickup line - get sessionnumber and from	
				($session, $from) = ($line =~ /.*\[\d*\]: (\S*):.*from=<(\S*)>/);
				
				$arfrom {$session} = $from;
				$armonth {$session} = $month;
				$arday {$session} = $day;
				$arhour {$session} = $hour;
				
				next LOOP;
			}
			
			if ($linetype eq "qmgr") { 
			
				$nqmgr++;
				# Qmgr line - get sessionnumber, size and number of recipients	
				($session, $size, $recipients) = ($line =~ /.*\[\d*\]: (\S*):.*size=(\d*), nrcpt=(\d*)/);
								
				$arsize {$session} = $size;
				$arrecipients {$session} = $recipients;
				
				next LOOP;
			}
		
			if ( $linetype eq "smtp") { 
			
				$nsmtp++;
				# smtp line - get sessionnumber, to, delay and status	
				($session, $to, $delay, $status) = ($line =~ /.*\[\d*\]: (\S*):.*to=<(\S*)>.*delay=(\d*), status=(\w*) /);
				
				$arto {$session} = $to;
				$ardelay {$session} = $delay;
				$arstatus {$session} = $status;
				
				next LOOP;
			}
			
			$nskipped++;
		
		}
	}
		
	$enddate = "$month/$day $hour:$minute";

	$n = $.;
	
	close LOGFILE or die $dieclosemsg;
		
	# Generate tables
	foreach my $session (keys %arfrom) {
	
		#DEBUG! print "$session ";
			
		# Check if all fields are defined
		next if (!(defined $arto {"$session"}));
		next if (!(defined $ardelay {"$session"}));
		next if (!(defined $arstatus {"$session"}));
		next if (!(defined $arsize {"$session"}));
		next if (!(defined $arrecipients {"$session"}));
			
		$nmessages++;
			
		#populate variables
		$month = $armonth {"$session"};
		$day = $arday {"$session"};
		$hour = $arhour {"$session"};
		$from = $arfrom {"$session"};
		$to = $arto {"$session"};
		$size = int ($arsize {"$session"} / 1024); # KB values
		$delay = int ($ardelay {"$session"} / 60); # Minutes
		$status = $arstatus {"$session"};
		$recipients = $arrecipients {"$session"};
						
		GenerateTables ();
	}
}