#########
#
# SNORT module for log reporting system
# v1.0 - Initial version, 06.2002, Tevfik K. (www.itefix.no)
 
# Sets analysis parameters - specific for each module
@dim = ( "month", "day", "hour", "from", "to", "event");

# Get common functions & parameters
do "lib1.pl";

################
# MAIN PROGRAM #
################

SetCommonParameters ();

# initialize counters
$n = $nalert = $nportdetect = $nportstatus = $nportend = 0;

ProcessSnortLogFile ($logfilename);

WriteIndexPage (
	"SNORT Logfile analysis",
	"Alerts Analysis Table",
	"Number of alerts: ", "$nalert",
	"Portscanning messages: ", "detect - $nportdetect, status - $nportstatus, end - $nportend"
	);

WriteDetailsPages ();

###############
# SUBROUTINES #
###############

# ProcessSnortLogFile : 
#	Processes snort logfile and populates counter arrays
#	Parameter : Snort log filename
#
sub ProcessSnortLogFile {

	my $snortfile = shift;
		
	open LOGFILE, $snortfile or die $dieopenmsg;

LOOP: 
	while (<LOGFILE>) {

		my $line = $_;
		
		($month, $day, $hour, $minute) = ($line =~ m"(\d\d)/(\d\d)-(\d\d):(\d\d)");
		
		($startdate = "$month/$day $hour:$minute") if ($. == 1);
				
		#DEBUG! 
		print "$. " if  ( ( ($./5000) - int($./5000) ) == 0) ;

		do { $nportstatus++ ; next LOOP } if /portscan status/;
		do { $nportdetect++ ; next LOOP } if /PORTSCAN DETECTED/;

		do {
			$nportend++;
			($month, $day, $hour, $from, $totaltime, $totalhosts, $totaltcp, $totaludp) =
				($line =~ m"(\d\d)/(\d\d)-(\d\d).*from (.*): TOTAL time\((.*)\) hosts\((.*)\) TCP\((.*)\) UDP\((.*)\)");
			next LOOP;
		} if /End of portscan/;

		$nalert++;
		
		# parse log line
		($month, $day, $hour, $event, $from, $to) = ($line =~ m"(\d\d)/(\d\d)-(\d\d).*\].*\]\s(.*)\s\[\*\*\].*\{.*\}\s(.*)\s->\s(.*)");
	
		# remove port part from from/to adressess if necessary
		if (($l = index ($from, ":")) > 0) { $from = substr ($from, 0, $l) };
		if (($l = index ($to, ":")) > 0) { $to = substr ($to, 0, $l) };
				
		#DEBUG! print "$month $day $hour $event $from $to\n";
		
		# Generate tables
		GenerateTables ();
						
	}
	
	$enddate = "$month/$day $hour:$minute";

	$n = $.;
	
	close LOGFILE or die $dieclosemsg;
}

