#########
#
# Squid module for log reporting system
# v1.0 - Initial version, 06.2002, Tevfik K. (www.itefix.no)

# Sets analysis parameters - specific for each module
@dim = ( "month", "day", "hour", "elapsed", "from", "cache", "result", "size", "method", "url", "content");

# Get common functions & parameters
do "lib1.pl";

################
# MAIN PROGRAM #
################

SetCommonParameters ();

ProcessSquidLogFile ($logfilename);

WriteIndexPage (
	"SQUID Logfile analysis",
	"Cache Analysis Table"
	);

WriteDetailsPages ();

###############
# SUBROUTINES #
###############

# ProcessSquidLogFile : 
#	Processes squid logfile and populates counter arrays
#	Parameter : squid log filename
#
sub ProcessSquidLogFile {

	my $logfile = shift;
		
	open LOGFILE, $logfile or die $dieopenmsg;

LOOP: 
	while (<LOGFILE>) {

		my $line = $_;
		
		($timetag, $elapsed, $from, $cache, $result, $size, $method, $fullurl, $content) = 
			($line =~ m"([\d\.]+)\s+(\d+) ([\d\.]+) (\w+)/(\d+) (\d+) (\w+) (\S+) \S+ \S+ (\S+)");

		# extract time info
		($tmp, $minute, $hour, $day, $month, $tmp, $tmp, $tmp) = gmtime ($timetag);

		# parse full url
		($tmp, $tmp, $url) = split (m"/", $fullurl);
		
		# elapsed time in integer seconds
		$elapsed = int ($elapsed / 1000);
		
		# size in integer KB
		$size = int ($size / 1024);
				
		($startdate = "$month/$day $hour:$minute") if ($. == 1);
				
		print "$. " if  ((($./1000) - int($./1000) ) == 0) ;
		
		#DEBUG!	print "$day, $month, $hour, $elapsed, $from, $cache, $result, $size, $method, $url, $content\n";
		
		GenerateTables ();
	}
	
	$enddate = "$month/$day $hour:$minute";

	$n = $.;
	
	close LOGFILE or die $dieclosemsg;
}
